/*
   AngelCode Tool Box Library
   Copyright (c) 2007 Andreas Jnsson
  
   This software is provided 'as-is', without any express or implied 
   warranty. In no event will the authors be held liable for any 
   damages arising from the use of this software.

   Permission is granted to anyone to use this software for any 
   purpose, including commercial applications, and to alter it and 
   redistribute it freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you 
      must not claim that you wrote the original software. If you use
      this software in a product, an acknowledgment in the product 
      documentation would be appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and 
      must not be misrepresented as being the original software.

   3. This notice may not be removed or altered from any source 
      distribution.
  
   Andreas Jnsson
   andreas@angelcode.com
*/

#ifndef FONT_H
#define FONT_H

#include <vector>
#include <string>

struct IDirect3DTexture9;

namespace acGraphics
{
class CDynRender;

struct SCharDescr
{
	SCharDescr() : srcX(0), srcY(0), srcW(0), srcH(0), xOff(0), yOff(0), xAdv(0), page(0) {}

	short srcX;
	short srcY;
	short srcW;
	short srcH;
	short xOff;
	short yOff;
	short xAdv;
	short page;
	unsigned int chnl;

	std::vector<short> kerningPairs;
};

class CFont
{
public:
	CFont();
	~CFont();

	int Init(const char *fontFile, CDynRender *dynRender);
	float GetTextWidth(const char *text, int count);
	void Write(float x, float y, float z, const char *text, int count, unsigned int mode);
	void WriteML(float x, float y, float z, const char *text, int count, unsigned int mode);

	void SetHeight(float h);
	float GetHeight();

	float GetBottomOffset();
	float GetTopOffset();

	void PreparePixelPerfectOutput();

protected:
	int SkipWhiteSpace(std::string &str, int start);
	int FindEndOfToken(std::string &str, int start);

	void InterpretCommon(std::string &str, int start);
	void InterpretChar(std::string &str, int start);
	void InterpretSpacing(std::string &str, int start);
	void InterpretKerning(std::string &str, int start);
	void InterpretPage(std::string &str, int start, const char *fontFile);

	void InternalWrite(float x, float y, float z, const char *text, int count);

	float AdjustForKerningPairs(int first, int second);

	short fontHeight; // total height of the font
	short base;       // y of base line
	short scaleW;
	short scaleH;
	SCharDescr *chars[256];
	SCharDescr defChar;

	float scale;

	CDynRender *render;

	std::vector<IDirect3DTexture9*> pages;
};

const int FONT_ALIGN_LEFT   = 0;
const int FONT_ALIGN_CENTER = 1;
const int FONT_ALIGN_RIGHT  = 2;

}

#endif