/*

    This effect will simply copy the texture to the triangle, with no lighting or other modifications

*/

shared float4x4 g_mWorld : WORLD;      // World matrix
shared float4x4 g_mView  : VIEW;       // View matrix
shared float4x4 g_mProj  : PROJECTION; // Projection matrix

texture  g_txScene         : TEXTURE0;

sampler2D g_samScene =
sampler_state
{
    Texture = <g_txScene>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
};

struct VS_OUTPUT
{
    float4 Position   : POSITION;   // vertex position 
    float2 TextureUV  : TEXCOORD0;  // vertex texture coords 
};
VS_OUTPUT VertScene( float4 vPos         : POSITION,
                     float2 vTex0        : TEXCOORD0 )
{
    VS_OUTPUT Out;

    // Transform the position from object space to homogeneous projection space
    Out.Position = mul( vPos, g_mWorld );
    Out.Position = mul( Out.Position, g_mView );
    Out.Position = mul( Out.Position, g_mProj );

    // Just copy the texture coordinate through
    Out.TextureUV = vTex0;

    return Out;
}


float4 PixScene( VS_OUTPUT In ) : COLOR0
{
    // Lookup color from texture
    return tex2D(g_samScene, In.TextureUV);
}


//--------------------------------------------------------------------------------------
// Techniques
//--------------------------------------------------------------------------------------
technique RenderScene
{
    pass P0
    {
		// Turn on alpha blending
		AlphaBlendEnable = true;
		DestBlend        = INVSRCALPHA;
		SrcBlend         = SRCALPHA;
		
		// Set the vertex and pixel shaders
        VertexShader = compile vs_2_0 VertScene();
        PixelShader  = compile ps_2_0 PixScene();
    }
}
