/*

  This effect will convert the pixel depth to the color, thus making it ideal for shadow map generation

*/

shared float4x4 g_mWorld : WORLD;      // World matrix
shared float4x4 g_mView  : VIEW;       // View matrix
shared float4x4 g_mProj  : PROJECTION; // Projection matrix

struct VS_OUTPUT
{
    float4 Position   : POSITION;   // vertex position 
    float2 Depth      : TEXCOORD0;  // z and w of vertex position
};
VS_OUTPUT VertScene( float4 vPos         : POSITION )
{
    VS_OUTPUT Out;

    // Transform the position from object space to homogeneous projection space
    Out.Position = mul( vPos, g_mWorld );
    Out.Position = mul( Out.Position, g_mView );
    Out.Position = mul( Out.Position, g_mProj );

    // Store z and w in the texture coordinate
    Out.Depth.xy = Out.Position.zw;

    return Out;
}


float4 PixScene( VS_OUTPUT In ) : COLOR0
{
    // Depth is z / w
    return In.Depth.x / In.Depth.y;
}


//--------------------------------------------------------------------------------------
// Techniques
//--------------------------------------------------------------------------------------
technique RenderScene
{
    pass P0
    {
        // Cull the front facing triangles, so that backfaces are used instead
        CullMode = 2; // 2 = Clock-wise, 3 = Counter clock-wise
        
		// Turn off alpha blending
		AlphaBlendEnable = false;
		
		// Set the vertex and pixel shaders
        VertexShader = compile vs_2_0 VertScene();
        PixelShader  = compile ps_2_0 PixScene();
    }
}
